/**
 * An example alert sender that waits for an interval...to demonstrate a problem.
 */
package com.ocsystems.rtiplugin;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.rhq.core.domain.alert.Alert;
import org.rhq.core.domain.alert.notification.SenderResult;
import org.rhq.core.domain.auth.Subject;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.operation.bean.ResourceOperationSchedule;
import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;
import org.rhq.enterprise.server.alert.AlertManagerLocal;
import org.rhq.enterprise.server.auth.SessionManager;
import org.rhq.enterprise.server.auth.SubjectManagerLocal;
import org.rhq.enterprise.server.operation.OperationManagerLocal;
import org.rhq.enterprise.server.plugin.pc.alert.AlertSender;
import org.rhq.enterprise.server.resource.ResourceManagerLocal;
import org.rhq.enterprise.server.util.LookupUtil;

public class MyAlertSender extends AlertSender<MyAlertComponent> 
{
   public static final String PREFIX = "PROBLEM: "; 
   private static final Log log = LogFactory.getLog(MyAlertSender.class);

   private static final long DELAY_MS = 5 * 60 * 1000;
   private static final String USER_NAME = "rhqadmin";
   
   private static final int DEFAULT_OPERATION_TIMEOUT = 240;
   private static final long DEFAULT_SESSION_TIMEOUT = 3000000L;

   @Override
   public SenderResult send(Alert alert) 
   {
      log.info(PREFIX + "send:");

      try
      {
         AlertManagerLocal alertManager = LookupUtil.getAlertManager();
         ResourceManagerLocal resourceManager = LookupUtil.getResourceManager();

         int resId = alert.getAlertDefinition().getResource().getId();
         
         SubjectManagerLocal subjectManager = LookupUtil.getSubjectManager();
         Subject subject = subjectManager.getSubjectByName(USER_NAME);
         if (subject == null) 
         {
             throw new IllegalArgumentException("User with name " + USER_NAME + " doesn't exist anymore.");
         }
         
         // login for 2 mins
         Subject subject2 = SessionManager.getInstance().put(subject, DEFAULT_SESSION_TIMEOUT);

         Configuration cfg = new Configuration();

         OperationManagerLocal operationManager = LookupUtil.getOperationManager();
         ResourceOperationSchedule schedule =
            operationManager.scheduleResourceOperation(
                  subject2, 
                  resId, 
                  "viewProcessList", 
                  0, 
                  0, 
                  0, 
                  DEFAULT_OPERATION_TIMEOUT, 
                  cfg, 
                  "test");

         //TODO fix this....
         //ResourceOperationHistory hist = waitForOperation(subject, schedule);
      
         // Wait
         log.info(PREFIX + "wait " + DELAY_MS);
         try
         {
            Thread.sleep(DELAY_MS);
         }
         catch (Exception e)
         {
            // ignore
         }
         
         return SenderResult.getSimpleSuccess(PREFIX + "success");
      } 
      catch (Throwable t) 
      {
         return SenderResult.getSimpleFailure(PREFIX + "failed - cause: " + t);
      }
   }

}
